/* Do not edit this file, make creates it. */
extern Char STR0[];
extern Char STR1[];
extern Char STRHOME[];
extern Char STRLANG[];
extern Char STRLC_CTYPE[];
extern Char STRLOGNAME[];
extern Char STRLbrace[];
extern Char STRLparen[];
extern Char STRLparensp[];
extern Char STRNULL[];
extern Char STRPATH[];
extern Char STRPWD[];
extern Char STRQNULL[];
extern Char STRRbrace[];
extern Char STRRparen[];
extern Char STRTERM[];
extern Char STRUSER[];
extern Char STRalias[];
extern Char STRampm[];
extern Char STRand2[];
extern Char STRand[];
extern Char STRaout[];
extern Char STRargv[];
extern Char STRbang[];
extern Char STRcaret[];
extern Char STRcdpath[];
extern Char STRcent2[];
extern Char STRcenthash[];
extern Char STRcentminus[];
extern Char STRcentplus[];
extern Char STRchase_symlinks[];
extern Char STRchild[];
extern Char STRcolon[];
extern Char STRcwd[];
extern Char STRdefault[];
extern Char STRdot[];
extern Char STRdotdotsl[];
extern Char STRdotsl[];
extern Char STRecho[];
extern Char STRequal[];
extern Char STRfakecom1[];
extern Char STRfakecom[];
extern Char STRfignore[];
extern Char STRfilec[];
extern Char STRhistchars[];
extern Char STRhistfile[];
extern Char STRhistory[];
extern Char STRhome[];
extern Char STRignore_symlinks[];
extern Char STRignoreeof[];
extern Char STRjobs[];
extern Char STRlistjobs[];
extern Char STRlogout[];
extern Char STRlong[];
extern Char STRmail[];
extern Char STRmh[];
extern Char STRminus[];
extern Char STRml[];
extern Char STRmn[];
extern Char STRmquestion[];
extern Char STRnice[];
extern Char STRnobeep[];
extern Char STRnoclobber[];
extern Char STRnoglob[];
extern Char STRnohup[];
extern Char STRnonomatch[];
extern Char STRnormal[];
extern Char STRnotify[];
extern Char STRor2[];
extern Char STRor[];
extern Char STRpath[];
extern Char STRprintexitvalue[];
extern Char STRprompt2[];
extern Char STRprompt[];
extern Char STRpushdsilent[];
extern Char STRsavehist[];
extern Char STRsemisp[];
extern Char STRshell[];
extern Char STRslash[];
extern Char STRsldotcshrc[];
extern Char STRsldotlogin[];
extern Char STRsldthist[];
extern Char STRsldtlogout[];
extern Char STRsource[];
extern Char STRsp3dots[];
extern Char STRspLarrow2sp[];
extern Char STRspLarrowsp[];
extern Char STRspRarrow2[];
extern Char STRspRarrow[];
extern Char STRspRparen[];
extern Char STRspace[];
extern Char STRspand2sp[];
extern Char STRspor2sp[];
extern Char STRsporsp[];
extern Char STRstar[];
extern Char STRstatus[];
extern Char STRsymcent[];
extern Char STRsymhash[];
extern Char STRterm[];
extern Char STRthen[];
extern Char STRtilde[];
extern Char STRtime[];
extern Char STRtmpsh[];
extern Char STRunalias[];
extern Char STRuser[];
extern Char STRverbose[];
extern Char STRwordchars[];
